//
//  MINTLColor2HTMLColor.swift
//  MenuTester
//
//  Created by narita on 2017/08/03.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

extension NSColor
{
    @objc var htmlColor: NSString
    {
        let theRGBColor = self.usingColorSpace(.genericRGB)!
        
        let r: NSInteger = NSInteger(theRGBColor.redComponent   * 255.0)
        let g: NSInteger = NSInteger(theRGBColor.greenComponent * 255.0)
        let b: NSInteger = NSInteger(theRGBColor.blueComponent  * 255.0)
        
        let str = NSString.init(format:"#%02lx%02lx%02lx", r, g, b);
        
        return str
    }
}

class MINTLColor2HTMLColor: ValueTransformer
{
    // MARK: - class methods
    
    class func registTransformer()
    {
        self.setValueTransformer(MINTLColor2HTMLColor(),
                                 forName: .MINTLColor2HTMLColorName)
    }

    override class func transformedValueClass() -> AnyClass
    {
        return NSString.classForCoder()
    }
    
    override class func allowsReverseTransformation() -> Bool
    {
        return true
    }

    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    override func transformedValue(_ value: Any?) -> Any?
    {
        guard let theValue: NSColor = value as? NSColor else
        {
            return "#000000"
        }
            
        return theValue.htmlColor
    }
    
    override func reverseTransformedValue(_ value: Any?) -> Any?
    {
        guard let theValue: NSString = value as? NSString else
        {
            return NSColor.black
        }

        if( theValue.hasPrefix("#") && theValue.length == 7 )
        {
            let r = theValue.substring(with:NSMakeRange(1, 2))
            let g = theValue.substring(with:NSMakeRange(3, 2))
            let b = theValue.substring(with:NSMakeRange(5, 2))

            var red:   UInt32 = 0
            var green: UInt32 = 0
            var blue:  UInt32 = 0
            
            Scanner.init(string:r).scanHexInt32(&red)
            Scanner.init(string:g).scanHexInt32(&green)
            Scanner.init(string:b).scanHexInt32(&blue)
            
            return NSColor.init(calibratedRed: CGFloat(red) / 255.0,
                                        green: CGFloat(green) / 255.0,
                                         blue: CGFloat(blue) / 255.0,
                                        alpha: 1.0)
        }
        else
        {
            return NSColor.black
        }
    }


    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    // MARK: - Utility methods
}

extension NSValueTransformerName
{
    static let MINTLColor2HTMLColorName = NSValueTransformerName(rawValue: "MINTLColor2HTMLColor")
}


class MINTLHTMLColor2Color: ValueTransformer
{
    // MARK: - class methods
    
    class func registTransformer()
    {
        self.setValueTransformer(MINTLHTMLColor2Color(),
                                 forName: .MINTLHTMLColor2ColorName)
    }

    override class func transformedValueClass() -> AnyClass
    {
        return NSColor.classForCoder()
    }
    
    override class func allowsReverseTransformation() -> Bool
    {
        return true
    }

    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    override func transformedValue(_ value: Any?) -> Any?
    {
        return self.color2HTML.reverseTransformedValue(value)
    }
    
    override func reverseTransformedValue(_ value: Any?) -> Any?
    {
        return self.color2HTML.transformedValue(value)
    }

    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    let color2HTML: MINTLColor2HTMLColor = MINTLColor2HTMLColor.init()
    
    // MARK: - Utility methods
}

extension NSValueTransformerName
{
    static let MINTLHTMLColor2ColorName = NSValueTransformerName(rawValue: "MINTLHTMLColor2Color")
}
