//
//  MINTLUserDefaultsController.swift
//  UndoDefault
//
//  Created by narita on 2019/07/12.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

// (1) subclassingではshardインスタンスを上書きできなかったので、Method Swizzlingで "values"の値を書き換える

// appliesImmediatelyはtrueにしておくこと。そうしないとUndoの意味がないし、生合成が取れなくなるはず。


extension NSUserDefaultsController
{
    // MARK: - class methods

    // MARK: - init methods

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // Undo可能なrevertToInitialValues
    @IBAction open func revertToInitialValuesWithUndo(_ sender: Any?)
    {
        initialValues?.forEach()
        {
            i in
            
            (self.values as AnyObject).setValue(i.value, forKey: i.key)
        }
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    static func classInitialize_MINTL()
    {
        let theNewSelector: Selector = NSSelectorFromString("pseudo_values")
        let theOldSelector: Selector = NSSelectorFromString("values")
        
        if self.exchangeInstanceMethod(from: theNewSelector, to: theOldSelector)
        {
            // 成功
        }
    }

    @objc func pseudo_values() -> Any
    {
        let theProxy = MINTLUserDefaultsControllerUndoHelper.shared
        theProxy.values = pseudo_values() as? NSObject
        
        return theProxy
    }
    
    // MARK: - Utility methods
    
    // plistで一部だけ初期化する。prefarenceWindowがTab構成になっている場合に使用する。
    open func resetValues(fromPlsitName: String)
    {
        guard let thePlistPath = Bundle.main.path(forResource: fromPlsitName, ofType: "plist") else
        {
            return
        }
        
        guard let theDict = NSDictionary(contentsOfFile: thePlistPath) as? [String: Any] else
        {
            return
        }
        
        for (key, value) in theDict
        {
            (self.values as AnyObject).setValue(value, forKey: key)
        }
    }
}

// (2) Undo用の関数を仕込むproxy

class MINTLUserDefaultsControllerUndoHelper: NSObject
{
    // MARK: - class methods
    
    static let shared: MINTLUserDefaultsControllerUndoHelper =
    {
        return MINTLUserDefaultsControllerUndoHelper()
    }()

    // MARK: - init methods
    
    // MARK: - dealloc
        
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc override func value(forKey key: String) -> Any?
    {
        return self.values!.value(forKey:key)
    }

    @objc override func setValue(_ value: Any?, forKey key: String)
    {
//        print("setValue \(key) : \(value)")

        // これがUndoバッファに貯める場所
        (self.undoManager!.prepare(withInvocationTarget: self) as AnyObject).setValue(self.value(forKey: key), forKey: key)

        self.values!.setValue(value, forKey: key)
    }
    
    override func forwardingTarget(for aSelector: Selector!) -> Any?
    {
//        print("forwardingTarget \(aSelector) ")

        return self.values
    }
    
    @objc var values: NSObject? = nil
    
    @objc var undoManager: UndoManager? = nil
    
    // MARK: - Utility methods
}
